from mediasort import setup_logging, setup_mime

from mediasort.classify import Classifier
from mediasort.classify.classification import Classification

from mediasort.act import Action

import logging
logger = logging.getLogger(__name__)

import click
import yaml

DEFAULT_CONFIG = {
    'log_path': 'mediasort.log',
    'log_level': 'info'
}

config_option = click.option('--config', '-c',
                             type=click.File(),
                             help='path to config.yml')
dry_run_option = click.option('--dry-run', '-d',
                              help='don\'t make any filesystem changes',
                              is_flag=True,
                              default=False)
quiet_option = click.option('--quiet', '-q',
                              help='Quieter operation',
                              is_flag=True,
                              default=False)
verbose_option = click.option('--verbose', '-v',
                              help='Quieter operation',
                              is_flag=True,
                              default=False)
path_option = click.argument('path')

@click.group()
@config_option
@quiet_option
@verbose_option
@click.version_option()
@click.pass_context
def cli(ctx, config, quiet, verbose):

    if quiet and verbose:
        raise click.ClickException("Can't set both --quiet and --verbose")

    conf = dict(DEFAULT_CONFIG)
    if config:
        conf.update(yaml.load(config))

    if verbose:
        conf['log_level'] = 'debug'
    if quiet:
        conf['log_level'] = 'warning'

    setup_logging(conf)
    setup_mime(conf)

    ctx.obj = dict(
        config=conf
    )

@cli.command()
@path_option
def test(ctx, path):
    pass

@cli.command()
@click.pass_context
def validate(ctx):
    errors = []

    config = ctx.obj['config']
    action_defs = config.get('actions', [])
    for index, action_def in enumerate(action_defs):
        errors.extend([
            "Action %s %s" % (index + 1, e)
            for e in Action.validate_definition(action_def)
        ])

    if errors:
        raise click.ClickException('Your configuration has errors:\n - %s' % (
            '\n - '.join(errors)
        ))


@cli.command()
@dry_run_option
@path_option
@click.pass_context
def classify(ctx, dry_run, path):
    config = ctx.obj['config']

    classifier = Classifier(config)

    if path.endswith('/'):
        path = path[:-1]

    rv = classifier.classify(path)

    logger.info("%s: %s" % (path, rv))

    for action_set in classifier.actions_for(rv):
        for action in Action.from_config(action_set, rv):
            action.perform(dry_run=dry_run)



import logging
logger = logging.getLogger(__name__)

import os
from xml.etree import ElementTree as ET
import requests

import click

from mediasort.act import Action
from mediasort.lib import as_list
from mediasort.lib import validate


class RefreshPlexSection(Action):
    '''
    refresh_plex_section:
        server_address: http://...
        section_name: ...
    '''

    __key__ = "refresh_plex_section"

    def validate_options(self):
        if 'section_name' not in self.options:
            raise click.ClickException('refresh_plex_section needs section_name')
        if 'server_address' not in self.options:
            raise click.ClickException('refresh_plex_section needs server_address')

    def perform(self, dry_run):
        section_path = 'library/sections'
        sections = ET.fromstring(self._plex_request(section_path))
        xpath = './/Directory[@title="%s"]' % self.options['section_name']
        section = sections.find(xpath)

        if not section:
            msg = "Couldn't find section '%s'" % self.options['section_name']
            raise click.ClickException(self.error_message(msg))

        if not dry_run:
            return self._plex_request(
                '/'.join([section_path, section.attrib['key'], 'refresh'])
            )


    def _plex_request(path, method='get', **data):
        request = getattr(requests, method.lower())
        url = '/'.join([self.options['server_address'], path])
        response = request(url, **data)

        try:
            response.raise_for_status()
        except e:
            raise click.ClickException(e.message)


        return response.content







import logging
logger = logging.getLogger(__name__)

import click

from mediasort.lib.validate import Validatable
from mediasort.lib import as_list
from mediasort.classify.classification import MEDIA_TYPES

registry = {}

NO_KEY = object()

class ActionMetaclass(type):
    def __new__(cls, name, bases, attrs):
        cls = type.__new__(cls, name, bases, attrs)
        key = attrs.get('__key__', NO_KEY)
        if key == NO_KEY:
            raise Exception("Need to supply __key__ with Action declaration")

        if key in registry:
            raise Exception("'%s' already taken: '%s'" % (key, registry[key]))

        registry[key] = cls
        return cls

class Action(Validatable):
    __metaclass__ = ActionMetaclass
    __key__ = None

    def __init__(self, options):
        self.options = options

    @classmethod
    def validate_definition(cls, definition):
        errors = []
        if 'media_type' not in definition:
            errors.append("is missing `media_type`")
        if definition['media_type'] not in MEDIA_TYPES:
            errors.append("has an invalid `media_type`")

        if 'confidence' not in definition:
            errors.append("is missing `confidence`")
        if not isinstance(definition['confidence'], (int, float)):
            errors.append("must provide `confidence` as a number")

        if 'perform' not in definition:
            errors.append("is missing `perform`")

        for perform in as_list(definition['perform']):
            for key, options in perform.items():
                options['target'] = None

                if key in registry:
                    action = registry[key](options)
                    try:
                        action.validate_options()
                    except click.ClickException, e:
                        errors.append("error: %s" % e.message)
                else:
                    errors.append("references unknown perform: '%s'" % key)

        return errors

    def error_message(self, msg):
        return "%s: %s" % (self.__key__, msg)


    @classmethod
    def from_config(cls, action_hash, classification):

        try:
            actions = [
                registry[action](options)
                for d in as_list(action_hash['perform'])
                for action, options in d.items()
            ]

            for action in actions:
                action.options['target'] = classification
                action.validate_options()

            return actions
        except KeyError, e:
            msg = "Available actions: %s" % registry.keys()
            logger.error(e.message)
            logger.error(msg)
            raise click.ClickException(msg)

    def perform(**kwargs):
        raise NotImplementedError()

    def validate_options(self):
        pass


from mediasort.act import copy_to
from mediasort.act import plex
